
(*******************************************************************

  Utility routines.

*******************************************************************)
exception Incompatible_array_sizes

(* map function (f x y) over the elements of arrays a1 (x) and a2 (y);
   return a new array with entries 
   f a1.(0) a2.(0) ; f a1.(1) a2.(1) ... f a1.(n-1) a2.(n-1)
   *)
let map2 f a1 a2 =
  let len1 = Array.length a1 in 
  if (len1 != Array.length a2) then raise Incompatible_array_sizes
  else  
    let ret_a = Array.create len1 (f a1.(0) a2.(0))  in
    for i=0 to (len1-1) do ret_a.(i) <- f a1.(i) a2.(i) done ; ret_a
;;

(* call (f x y) for all elements of a1 and a2; return () *)
let iter2 f a1 a2 =
  let len1 = Array.length a1 in 
  if (len1 != Array.length a2) then raise Incompatible_array_sizes
  else  
    for i=0 to (len1-1) do (f a1.(i) a2.(i) ) done ; ()
;;

(* sum (f x i) for i from imin to imax, inclusive *)
let sum f x imin imax = 
  let acc = ref 0.0 in
  for i  = imin to imax do 
    acc := !acc +. (f x i) 
  done ; !acc
;;

(* Produce an evenly spaced grid from a to b containing n points.
   Return result as array.   *)
let linspace a b n = 
  let map a b n i = 
    let fi = float i and fn = float n in 
    (b-.a) *. (fi/.fn) +. a 
  in
  Array.init n (map a b (n-1))
;;


(* Given 2 functions f and fa, 
   return the relative error in percent and the error -- both are
   signed quantities. *)
let rel_abs_err f fa = 
  let xarr = linspace 0.0001 0.999 24 in
  let f_arr = Array.map f xarr and 
      fa_arr = Array.map fa xarr in 
  let abs_err_arr = map2 (-.) f_arr fa_arr in
  let percent_err x y = x/.y*.100.0 in 
  let rel_err_arr = map2 percent_err  abs_err_arr  f_arr in 
  (rel_err_arr, abs_err_arr)
;;

